--
-- A simulation model of ORIC ATMOS hardware
-- Copyright (c) SEILEBOST - March 2006 - 2010
--
-- All rights reserved
--
-- Redistribution and use in source and synthezised forms, with or without
-- modification, are permitted provided that the following conditions are met:
--
-- Redistributions of source code must retain the above copyright notice,
-- this list of conditions and the following disclaimer.
--
-- Redistributions in synthesized form must reproduce the above copyright
-- notice, this list of conditions and the following disclaimer in the
-- documentation and/or other materials provided with the distribution.
--
-- Neither the name of the author nor the names of other contributors may
-- be used to endorse or promote products derived from this software without
-- specific prior written permission.
--
-- THIS CODE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-- AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
-- THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-- PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
-- LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-- CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-- SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-- INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-- CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-- ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-- POSSIBILITY OF SUCH DAMAGE.
--
-- You are responsible for any legal issues arising from your use of this code.
--
-- The latest version of this file can be found at: passionoric.free.fr
--
-- Email seilebost@free.fr
--
--
-- Revision list
--
-- version 001 2006/03/?? : initial release
-- version 002 2009/01/06 : suite
-- version 003 2009/03/22 : version sram (ram statique)
-- version 004 2009/11/17 : nettoyage code
-- version 005 2009/11/18 : ajout gestion clavier PS2
-- version 006 2009/11/19 : correction gestion clavier PS2
-- version 007 2009/11/20 : correction gestion clavier PS2
-- version 090 2010/02/01 : Working release. It's not running.

  library ieee;
  use ieee.std_logic_1164.all;
  use ieee.std_logic_arith.all;
  use ieee.std_logic_unsigned.all;
  LIBRARY WORK;
  use work.pack_oric_xilinx_prims.all;
  use work.pkg_oric.all;
  --use work.pack_t65.all;
  LIBRARY STD;
  USE STD.TEXTIO.ALL;

  
entity ORIC is
  port (
    -- Gestion Port Extension
    AD                : inout std_logic_vector(15 downto 0);
    MAPn              : in    std_logic;
    ROMDISn           : in    std_logic;
    IRQn              : in    std_logic;
    CLK_EXT           : out   std_logic;  -- 1 MHZ
    RW                : out   std_logic;
    IO                : out   std_logic;
    IOCONTROL         : in    std_logic;
    D                 : inout std_logic_vector(7 downto 0);
    RESETn            : in    std_logic;
    
    -- Gestion clavier
    PS2_CLK           : in    std_logic;
    PS2_DATA          : in    std_logic;

    -- Gestion K7
    K7_TAPEIN         : in    std_logic;
    K7_TAPEOUT        : out   std_logic;
    K7_REMOTE         : out   std_logic;
    K7_AUDIOOUT       : out   std_logic;
    
    -- Gestion Audio/Video
    AUDIO_OUT         : out   std_logic_vector(2 downto 0); -- le 19/11/2009 : 3 sorties son et non pas 4 !
    VIDEO_R           : out   std_logic;
    VIDEO_G           : out   std_logic;
    VIDEO_B           : out   std_logic;
    VIDEO_HSYNC       : out   std_logic;
    VIDEO_VSYNC       : out   std_logic;
    VIDEO_SYNC        : out   std_logic;

    -- Gestion Imprimante (PRINTER)
    PRT_DATA          : inout std_logic_vector(7 downto 0);
    PRT_STR           : out   std_logic;  -- strobe
    PRT_ACK           : in    std_logic;  -- ack
   
    -- Clk master    
    CLK_12            : in    std_logic; -- CLK MASTER
	 
	 -- DEBUG
	 DBG_ROM_DOUT      : out   std_logic_vector(7 downto 0);
	 DBG_ULA_AD        : out   std_logic_vector(15 downto 0)
    );
end;

architecture RTL of ORIC is

    signal reset_dll_h        : std_logic;
    signal loc_reset_sampled  : std_logic; --actif  1
	 signal loc_reset_sampledn : std_logic; --actif  0
    signal delay_count        : std_logic_vector(7 downto 0) := (others => '0');
    signal clk_cnt            : std_logic_vector(2 downto 0) := "000";
    
    -- CLOCK_INTERNE 12 MHz
    signal clk_in             : std_logic;

    -- cpu
    signal cpu_ad             : std_logic_vector(23 downto 0);
    signal cpu_di             : std_logic_vector(7 downto 0);
    signal cpu_do             : std_logic_vector(7 downto 0);
    signal cpu_rw             : std_logic;
    signal cpu_irq            : std_logic;
    signal cpu_nmi            : std_logic;
    --
    -- le 17/11/2009 signal loc_io_sel         : std_logic_vector(3 downto 0);
    -- le 17/11/2009 signal loc_blk_sel        : std_logic_vector(7 downto 0);
    -- le 17/11/2009 signal loc_ram_sel        : std_logic_vector(7 downto 0);

    -- ULA    
    -- le 17/11/2009 signal ula_oe_l           : std_logic;
    -- le 17/11/2009 signal ula_do             : std_logic_vector( 7 downto 0);
    -- le 17/11/2009 signal ula_di             : std_logic_vector(11 downto 0);
    signal ula_phi2           : std_logic;
    signal ula_CSIOn          : std_logic;
	 signal ula_CSIO           : std_logic;
    signal ula_CSROMn         : std_logic;
    signal ula_CSRAMn         : std_logic; -- add 05/02/09
      
    -- VIA    
    signal via_pa_out_oe      : std_logic_vector(7 downto 0);
    signal via_pa_in          : std_logic_vector(7 downto 0);
    signal via_pa_out         : std_logic_vector(7 downto 0);
    signal via_ca1_in         : std_logic;     
    signal via_ca2_in         : std_logic;
    -- le 17/11/2009 signal via_ca2_out        : std_logic;
    -- le 17/11/2009 signal via_ca2_oe_l       : std_logic;    
    -- le 17/11/2009 signal via_cb1_in         : std_logic;
    signal via_cb1_out        : std_logic;
    signal via_cb1_oe_l       : std_logic;
    signal via_cb2_in         : std_logic;
    signal via_cb2_out        : std_logic;
    signal via_cb2_oe_l       : std_logic;
    signal via_in             : std_logic_vector(7 downto 0);
    signal via_out            : std_logic_vector(7 downto 0);
    signal via_oe_l           : std_logic_vector(7 downto 0);
    signal via_dout           : std_logic_vector(7 downto 0);
    
    -- KEYBOARD
    -- 18/11/2009 signal kbd_col_out        : std_logic_vector(7 downto 0);
    -- 18/11/2009 signal kbd_col_in         : std_logic_vector(7 downto 0);
    -- 18/11/2009 signal kbd_col_oe_l       : std_logic_vector(7 downto 0);
    -- 18/11/2009 signal kbd_row_in         : std_logic_vector(7 downto 0);
    -- 18/11/2009 signal kbd_restore        : std_logic;
    signal KEY_COL			   : std_logic_vector(2 downto 0);
    signal KEY_ROWbit		   : std_logic_vector(7 downto 0);
    signal KPmask			      : std_logic_vector(7 downto 0);
    signal KP				      : std_logic;


    -- PSG
    signal psg_bdir           : std_logic; -- Lecture/Ecriture sur PSG
    
    -- user port
    -- le 17/11/2009 signal loc_pa             : std_logic_vector(7 downto 0); -- le port A du VIA en local
    -- le 17/11/2009 signal loc_pb             : std_logic_vector(7 downto 0); -- le port B du VIA en local
    signal ula_DB             : std_logic_vector(7 downto 0);
	 signal lSRAM              : std_logic_vector(7 downto 0);
    signal ula_AD_RAM         : std_logic_vector(7 downto 0);
    signal ula_AD_SRAM        : std_logic_vector(15 downto 0);
    signal ula_CE_SRAM        : std_logic;
    signal ula_OE_SRAM        : std_logic;
    signal ula_WE_SRAM        : std_logic;
	 signal ula_LATCH_SRAM     : std_logic;
    signal ula_CLK_4          : std_logic;
    signal ula_RASn           : std_logic;
    signal ula_CASn           : std_logic;
    signal ula_MUX            : std_logic;
    signal ula_RW_RAM         : std_logic;
    -- 17/11/2009 signal clk_4              : std_logic;
    -- 17/11/2009 signal clk_1              : std_logic;
    signal ENA_1MHZ           : std_logic;
    signal char_rom_dout      : std_logic_vector(7 downto 0);
	 
    -- gestion ram
    -- le 17/11/2009 signal ram_d_in           : std_logic_vector(7 downto 0);
    -- le 17/11/2009 signal ram_d_out          : std_logic_vector(7 downto 0);
	 
    --attribute CLKDV_DIVIDE : string;
    --attribute CLKDV_DIVIDE of CLKDLLA : label is "9";
	 
    -- le 17/11/2009 signal addr : std_logic_vector(15 downto 0);
begin

  -- Gestion clock
  clk_in <= CLK_12;
  clk_ext <= ula_phi2;
  
  -- Gestion Reset
  loc_reset_sampled  <= RESETn;
  loc_reset_sampledn <= not RESETn;

  ------------------------------------------------------------
  -- GESTION CPU 6502
  ------------------------------------------------------------
  cpu : T65
      port map (
          Mode    => "00",
          Res_n   => loc_reset_sampled,
          Enable  => '1',
          Clk     => ula_phi2,
          Rdy     => '1',
          Abort_n => '1',
          IRQ_n   => cpu_irq,
          NMI_n   => cpu_nmi,
          SO_n    => '1',
          R_W_n   => cpu_rw,
          Sync    => open,
          EF      => open,
          MF      => open,
          XF      => open,
          ML_n    => open,
          VP_n    => open,
          VDA     => open,
          VPA     => open,
          A       => cpu_ad,
          DI      => cpu_di,
          DO      => cpu_do
      );

 rom : entity work.rom_oa
  port map (
     clk => CLK_12,
--	  ENA => ula_CSROMn,
	  ADDR => cpu_ad(13 downto 0),
	  DATA => char_rom_dout
  );
  
  ------------------------------------------------------------
  -- GESTION ULA
  ------------------------------------------------------------
  ulag : ULA
    port map (
      CLK        => CLK_IN,
      PHI2       => ula_PHI2,
      CLK_4      => ula_CLK_4, -- Change name !!
      RW         => cpu_rw,
      RESETn     => loc_reset_sampled,
      MAPn       => MAPn,
      DB         => ula_DB,
      AD         => cpu_ad(15 downto 0),
      AD_RAM     => ula_AD_RAM,
      AD_SRAM    => ula_AD_SRAM,
		OE_SRAM    => ula_OE_SRAM,
		CE_SRAM    => ula_CE_SRAM,
		WE_SRAM    => ula_WE_SRAM,
		LATCH_SRAM => ula_LATCH_SRAM,
      RASn       => ula_RASn,
      CASn       => ula_CASn,
      MUX        => ula_MUX,
      RW_RAM     => ula_RW_RAM,
      CSIOn      => ula_CSIOn,
      CSROMn     => ula_CSROMn,
      CSRAMn     => ula_CSRAMn,
      R          => VIDEO_R,
      G          => VIDEO_G,
      B          => VIDEO_B,
      SYNC       => VIDEO_SYNC                 
      );
      
  ------------------------------------------------------------
  -- GESTION VIA
  ------------------------------------------------------------
  ula_CSIO <= not(ula_CSIOn);
  via : M6522
    port map (
      RS              => cpu_ad(3 downto 0),
      DATA_IN         => cpu_do(7 downto 0),
      DATA_OUT        => via_dout,
      DATA_OUT_OE_L   => open,

      RW_L            => cpu_rw,
      CS1             => ula_CSIO,
      CS2_L           => IOCONTROL,

      IRQ_L           => cpu_irq, -- note, not open drain

      -- PORT A
      CA1_IN          => via_ca1_in, -- PRT_ACK, 03/11/09
      CA2_IN          => via_ca2_in, -- psg_bdir, 03/11/09
      CA2_OUT         => psg_bdir,    -- via_ca2_out, 03/11/09
      CA2_OUT_OE_L    => open,

      PA_IN           => via_pa_in,
      PA_OUT          => via_pa_out,
      PA_OUT_OE_L     => via_pa_out_oe,

      -- PORT B
      CB1_IN          => K7_TAPEIN,
      CB1_OUT         => via_cb1_out,
      CB1_OUT_OE_L    => via_cb1_oe_l,

      CB2_IN          => via_cb2_in,
      CB2_OUT         => via_cb2_out,
      CB2_OUT_OE_L    => via_cb2_oe_l,

      PB_IN           => via_in,
      PB_OUT          => via_out,
      PB_OUT_OE_L     => via_oe_l,

      -- AUTRE
      RESET_L         => loc_reset_sampled,
      P2_H            => ula_phi2,
      CLK_4           => ula_CLK_4
      );

  ------------------------------------------------------------
  -- GESTION PSG AY-3-8192
  ------------------------------------------------------------    
  psg : AY3819X
    port map ( 
      DATA_IN   => via_pa_out,
      DATA_OUT  => via_pa_in,
      O_DATA_OE_L => open,
      RESET     => loc_reset_sampledn,
      CLOCK     => ula_phi2,
		CLOCK_DAC => CLK_IN,
      BDIR      => via_cb2_out,
      BC1       => psg_bdir, -- le 05/09/11 via_ca2_out,
      BC2       => '1',
      IOA       => open, -- le 18/11/09 kbd_COL_out,
      IOB       => open,
      AnalogA   => AUDIO_OUT(0),
      AnalogB   => AUDIO_OUT(1),
      AnalogC   => AUDIO_OUT(2)
    );

  
  ------------------------------------------------------------
  -- GESTION KEYBOARD
  ------------------------------------------------------------
  KEY : keyboard 
   port map(
	CLK		=> CLK_IN,--CLK24000,
	RESET		=> loc_reset_sampledn, -- reset actif à 1 !! 20/11/2009

	PS2CLK	=> PS2_CLK,
	PS2DATA	=> PS2_DATA,

	COL		=> KEY_COL,
	ROWbit	=> KEY_ROWbit
  );

  -- Keyboard
  KEY_COL <= via_out(2 downto 0);
  KPmask <= not( KEY_ROWbit ) and not( VIA_PA_OUT );
  KP <= '0' when KPmask = "00000000" else '1';
  via_in <= "1111" & KP & "111";
  
  ------------------------------------------------------------
  -- GESTION SRAM
  ------------------------------------------------------------
  ramv : entity work.sram
  port map
  (
   A     => ula_AD_SRAM,
	nOE	=> ula_OE_SRAM,
	nWE	=> ula_WE_SRAM,		
	nCE1	=> ula_CE_SRAM,
	nUB1	=> '1',
	nLB1	=> '0',
	D     => lSRAM
   );
	
  ad <= ula_ad_sram when ula_phi2 = '0' else cpu_ad(15 downto 0);
  
  ------------------------------------------------------------
  -- GESTION PORT K7
  ------------------------------------------------------------
  K7_TAPEOUT  <= via_out(7);
  K7_REMOTE   <= via_out(6);
  
  ------------------------------------------------------------
  -- GESTION PORT IMPRIMANTE
  ------------------------------------------------------------
  PRT_DATA    <= via_pa_out;
  PRT_STR     <= via_out(4);
  
  ------------------------------------------------------------
  -- GESTION BUS DE DONNEES ENTRE COMPOSANT
  ------------------------------------------------------------
  
  -- Multiplexage CPU et RAM/VIA et ROM
  u_cpu_read_mux : process(ula_phi2, ula_csromn, ula_LATCH_SRAM, via_dout, d, char_rom_dout)	
  begin
  -- Accs au Via
  if (ula_CSIOn = '0' and ula_LATCH_SRAM = '0' and IOCONTROL = '0' and cpu_rw = '1') then
     cpu_di <= via_dout;
  -- Accs au port d'extension
  elsif (ula_CSIOn = '0' and IOCONTROL = '1' and cpu_rw = '1') then
     cpu_di <= D;
  -- Accs ROM
  elsif (ula_csromn = '0' and IOCONTROL = '0') then
     cpu_di <= char_rom_dout;
  elsif (ula_LATCH_SRAM = '0' and IOCONTROL = '0') then
     -- Read data
     if (cpu_rw = '1') then
        cpu_di <= lSRAM;
	  end if;
  else
      cpu_di <= cpu_di;
  end if;
  end process;
  
  -- Multiplexage CPU et RAM
  lSRAM <= CPU_DO when ula_WE_SRAM = '0' else "ZZZZZZZZ";
  ULA_DB <= lSRAM;
 
  -- 26/10/2009 : Gestion du via, port B en entree PB3 : retour test keyboard
  -- 18/11/2009 : Gestion clavier par port PS2
  --via_in <= "00000000";
   
  -- Pour le bon fonctionnement 06/02/09 : A enlever
  cpu_nmi <= '1';
  -- Pour le bon fonctionnement 03/11/09 : A enlever
  via_ca1_in <= '1';
  via_ca2_in <= '1';
  via_cb2_in <= '1';
  
  -- Pour les signaux de sortie 19/11/09
  RW <= cpu_rw;
  IO <= ula_CSIOn;
  K7_AUDIOOUT  <= '0'; -- A relier  la sortie du PSG pour le son 
                       -- C'est purement artificiel il faudra
							  -- le gnrer  partir du filtrage des
							  -- signaux fournis par le PSG
  VIDEO_HSYNC <= '0'; -- juste pour les tests A remplacer par le doubler VGA
  VIDEO_VSYNC <= '0'; -- juste pour les tests A remplacer par le doubler VGA
							  
  
  -- debug :
  DBG_ROM_DOUT <= char_rom_dout;
  DBG_ULA_AD   <= ula_ad_sram;
  -- 26/11/2009
  process(loc_reset_sampled,ula_csromn )
  begin
  if (loc_reset_sampled = '0') then
   assert false report "RESET" severity warning;
  elsif (ula_csromn = '0') then
    -- FA14
    if (cpu_ad(15 downto 0) = x"FA14") then 
       assert false report "DEBUT TEST MEMOIRE" severity warning;
	 end if;
    if (cpu_ad(15 downto 0) = x"FA85") then 
       assert false report "FIN TEST MEMOIRE" severity warning;
	 end if;
	 if (cpu_ad(15 downto 0) = x"F8B8") then 
       assert false report "DEBUT CONFIGURER SYSTEME" severity warning;
	 end if;
	 if (cpu_ad(15 downto 0) = x"F8C8") then 
       assert false report "PENDANT CONFIGURER SYSTEME" severity warning;
	 end if;
	 if (cpu_ad(15 downto 0) = x"F9C9") then 
       assert false report "PENDANT CONFIGURER SYSTEME : Appel TEXT" severity warning;
	 end if;
	 if (cpu_ad(15 downto 0) = x"F9CB") then 
       assert false report "TEXT : appel SET ATTRIBUT" severity warning;
	 end if;
	 if (cpu_ad(15 downto 0) = x"EE34") then 
       assert false report "DEBUT IRQ" severity warning;
	 end if;
	 if (cpu_ad(15 downto 0) = x"EE39") then 
       assert false report "PENDANT IRQ" severity warning;
	 end if;
	  if (cpu_ad(15 downto 0) = x"EE8B") then 
       assert false report "FIN IRQ" severity warning;
	 end if;
	 --if (cpu_ad(15 downto 0) = x"EEC9") then 
     --  assert false report "DEBUT ATTENDRE Qu'UN TIMER S'ANNULE" severity warning;
	 --end if;
	 --if (cpu_ad(15 downto 0) = x"EECC") then 
    --   assert false report "LIRE TIMER ATTENDRE Qu'UN TIMER S'ANNULE" severity warning;
	 --end if;
	 --if (cpu_ad(15 downto 0) = x"EECF") then 
    --   assert false report "Is_0 ATTENDRE Qu'UN TIMER S'ANNULE" severity warning;
	 --end if;
	 if (cpu_ad(15 downto 0) = x"EED7") then 
       assert false report "FIN ATTENDRE Qu'UN TIMER S'ANNULE" severity warning;
	 end if;
	 --if (cpu_ad(15 downto 0) = x"EEAB") then 
    --   assert false report "DEBUT ECRIRE DANS UN TIMER " severity warning;
	 --end if;
	 --if (cpu_ad(15 downto 0) = x"EEC8") then 
     --  assert false report "FIN ECRIRE DANS UN TIMER " severity warning;
	 --end if;
	 if (cpu_ad(15 downto 0) = x"FA13") then 
       assert false report "FIN SET ATTRIBUT" severity warning;
	 end if;
	  if (cpu_ad(15 downto 0) = x"FA00") then 
       assert false report "TEXT : appel #238" severity warning;
	 end if;
	 if (cpu_ad(15 downto 0) = x"FA06") then 
       assert false report "FIN TEXT" severity warning;
	 end if;
	 if (cpu_ad(15 downto 0) = x"F8D0") then 
       assert false report "SUITE CONFIGURER SYSTEME" severity warning;
	 end if;
	 if (cpu_ad(15 downto 0) = x"F8D2") then 
       assert false report "SUITE CONFIGURER SYSTEME" severity warning;
	 end if;
	 if (cpu_ad(15 downto 0) = x"F8DB") then 
       assert false report "FIN CONFIGURER SYSTEME : caractere rom" severity warning;
	 end if;
	 if (cpu_ad(15 downto 0) = x"F8AF") then 
       assert false report "APPEL COLD START" severity warning;
	 end if;
	 if (cpu_ad(15 downto 0) = x"C597") then 
       assert false report "Touche pressee" severity warning;
	 end if;
	 --if (cpu_ad(15 downto 0) = x"C5E8") then 
    --   assert false report "BOUCLE ATTENTE SAISIE CLAVIER" severity warning;
	 --end if;
  end if;
  end process;
  
end RTL;
