--
-- A simulation model of ULA
-- Copyright (c) seilebost - 2001 - 2010
--
-- All rights reserved
--
-- Redistribution and use in source and synthezised forms, with or without
-- modification, are permitted provided that the following conditions are met:
--
-- Redistributions of source code must retain the above copyright notice,
-- this list of conditions and the following disclaimer.
--
-- Redistributions in synthesized form must reproduce the above copyright
-- notice, this list of conditions and the following disclaimer in the
-- documentation and/or other materials provided with the distribution.
--
-- Neither the name of the author nor the names of other contributors may
-- be used to endorse or promote products derived from this software without
-- specific prior written permission.
--
-- THIS CODE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-- AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
-- THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-- PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
-- LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-- CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-- SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-- INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-- CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-- ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-- POSSIBILITY OF SUCH DAMAGE.
--
-- You are responsible for any legal issues arising from your use of this code.
--
-- The latest version of this file can be found at: www.fpgaarcade.com
--
-- Email seilebost@free.fr
--
--
--
--  ula.vhd
--
--  Top of ULA
--
--        Copyright (C)2001 - 2010 SEILEBOST
--                   All rights reserved.
--
-- Revision list
--
-- $Id: top.vhd, v0.03 2007/08/22 00:00:00 SEILEBOST $
--
-- MODIFICATION :
--   v0.01 : 200X/??/??
--   v0.02 : 2005/??/??
--   v0.03 : 2007/08/22 : bus bidirectionnel => bus unidirectionnel
--   v0.04 : 2009/01/26 : Transformation ram dynamique en ram statique
--   v0.10 : 2009/06/25 : Intgration de la partie multiplexage de l'accès ram
--   v0.20 : 2010/02/01 : Release working
-- TODO :
-- Remark :

library ieee;
use ieee.std_logic_1164.all;
library work;
use work.pack_ula.all;

entity ula is
  port (CLK        :   in  std_logic;                     -- 12 MHz                       -- pin 07
        PHI2       :   out std_logic;                     -- 1 MHz CPU & system           -- pin 14
	     CLK_4      :   out std_logic;                     -- 4 MHz interne
        RW         :   in  std_logic;                     -- R/W from CPU                 -- pin 27
        RESETn     :   in  std_logic;                     -- RESET master                 -- none exist
        MAPn       :   in  std_logic;                     -- MAP                          -- pin 26
        DB         :   in  std_logic_vector( 7 downto 0); -- DATA BUS                     -- pin 8,17,11,12,13,4,34 
        AD         :   in  std_logic_vector(15 downto 0); -- ADDRESS BUS                  -- pin 32,33,31,15,35,22,29,30, A,B,C,D,E,F,G,H
        AD_RAM     :   out std_logic_vector( 7 downto 0); -- ADDRESS BUS for dynamic ram  -- pin 36,37,4,4,2,10,39
		  -- sram
        AD_SRAM    :   out std_logic_vector(15 downto 0); -- ADDRESS BUS for static ram
		  OE_SRAM    :   out std_logic;
		  CE_SRAM    :   out std_logic;
		  WE_SRAM    :   out std_logic;
		  LATCH_SRAM :   out std_logic;
		  -- ram dynamic
        RASn       :   out std_logic;                     -- RAS for dynamic ram          -- pin 10
        CASn       :   out std_logic;                     -- CAS for dynamic ram          -- pin 09
        MUX        :   out std_logic;                     -- Mux command                  -- pin 01
        RW_RAM     :   out std_logic;                     -- Read/Write for dynamic ram   -- pin 28
        CSIOn      :   out std_logic;                     -- Chip select IO (VIA)         -- pin 25
        CSROMn     :   out std_logic;                     -- ROM select                   -- pin 23
	     CSRAMn     :   out std_logic;                     -- RAM select
        R          :   out std_logic;                     -- Red                          -- pin 21
        G          :   out std_logic;                     -- Green                        -- pin 20
        B          :   out std_logic;                     -- Blue                         -- pin 19
        SYNC       :   out std_logic                      -- Synchronisation              -- pin 16        
        );
end;

-- the description of the logic --------------------------

architecture RTL of ula is

-- Signal CLOCK
signal CLK_12_EXT     : std_logic;                  -- CLOCK 12 MHz external
signal CLK_12_INT     : std_logic;                  -- CLOCK 12 MHz internal
signal CLK_24_INT     : std_logic;                  -- CLOCK 24 MHz internal
signal CLK_4_INT      : std_logic;                  -- CLOCK  4 MHz internal
signal CLK_1_INT      : std_logic;                  -- CLOCK  1 MHz internal
signal CLK_PIXEL_INT  : std_logic;                  -- CLOCK PIXEL  internal 
signal CLK_FLASH      : std_logic;                  -- CLOCK FLASH  external

-- Data Bus Internal
signal DATA_BUS_INT   : std_logic_vector(7 downto 0);

-- Manage memory access
signal VAP1       : std_logic_vector(15 downto 0);  -- VIDEO ADDRESS PHASE 1
signal VAP2       : std_logic_vector(15 downto 0);  -- VIDEO ADDRESS PHASE 2
signal BAC        : std_logic;                      -- BUS ADDRESS COLUMN
signal BAL        : std_logic;                      -- BUS ADDRESS LATCH
signal BAP        : std_logic_vector(15 downto 0);  -- BUS ADDRESS PROCESSOR 
signal VA1C       : std_logic;                      -- VIDEO ADDRESS PHASE 1 COLUMN
signal VA1L       : std_logic;                      -- VIDEO ADDRESS PHASE 1 LATCH
signal VA1R       : std_logic;                      -- VIDEO ADDRESS PHASE 1 ROW
signal VA2C       : std_logic;                      -- VIDEO ADDRESS PHASE 2 COLUMN
signal VA2L       : std_logic;                      -- VIDEO ADDRESS PHASE 2 LATCH 
signal VA2R       : std_logic;                      -- VIDEO ADDRESS PHASE 2 ROW
signal RAS        : std_logic;                      -- ROW  ADDRESS SIGNAL
signal CAS        : std_logic;                      -- COLUMN ADDRESS SIGNAL
signal RW_RAM_INT : std_logic;                      -- Read/Write INTERNAL TO DYNAMIC RAM
signal RW_INT     : std_logic;                      -- Read/Write INTERNAL FROM CPU
signal MUX_INT    : std_logic;                      -- MULTIPLIXER SIGNAL

-- local signal
signal txthir_sel   : std_logic;                     -- TXT/HIRES SELECT
signal txthir_dec   : std_logic;                     -- TXT/HIRES DECODE
signal dblhgt_sel   : std_logic;                     -- Double Height SELECT
signal forcetxt     : std_logic;                     -- Force text mode
signal isAttrib     : std_logic;                     -- Attrib
-- 17/11/2009 : not used signal load_sr      : std_logic;                     -- Load shift register video
signal attrib_dec   : std_logic;                     -- Attrib decode
signal ld_reg_0     : std_logic;                     -- Load zero into video register
signal reld_reg     : std_logic;                     -- Reload from register to shift
signal databus_en   : std_logic;                     -- Data bus enable
signal COMPSYNC_INT : std_logic;                     -- Composite Synchronization for video
signal blankingn    : std_logic;                     -- Blanking signal
-- 17/11/2009 : not used signal flash_sel    : std_logic;                     -- Flash mode SELECT
signal reload_sel   : std_logic;                     -- reload register SELECT
signal freq_sel     : std_logic;                     -- Frequency video SELECT (50 or 60 Hz)
signal ldfrombus    : std_logic;                     -- Load from Bus Data
-- 17/11/2009 : not used signal ldzero       : std_logic;                     -- Load Zero
signal chrowcnt     : std_logic_vector( 2 downto 0); -- ch?? row count
signal CPT_H        : std_logic_vector( 6 downto 0); -- Horizontal counter
signal CPT_V        : std_logic_vector( 8 downto 0); -- Vertical counter
signal l_AD_SRAM    : std_logic_vector(15 downto 0); -- local address sram

-- 09/02/09 : output enable for ram/rom
signal BAOE_int     : std_logic;

signal rgb_int      : std_logic_vector(2 downto 0);  -- Red Green Blue video signal

-- local select RAM, IO & ROM
signal CSRAMn_INT   : std_logic;              -- Chip Select Ram
signal CSIOn_INT    : std_logic;              -- Chip Select Input/Output
signal CSROMn_INT   : std_logic;              -- Chip select Rom

-- Bus Address internal
signal AD_RAM_INT   : std_logic_vector(15 downto 0);  -- BUS ADDRESS RAM

-- RESET internal 
signal RESETn_INT   : std_logic;

-- MAP internal 
signal MAPn_INT     : std_logic;

-- SRAM internal
signal SRAM_CE      : std_logic;
signal SRAM_OE      : std_logic;
signal SRAM_WE      : std_logic;
signal SRAM_LATCH   : std_logic;

begin

-- 04/02/09 : Inversion signal de decodage RAM/ROM
-- correctif : ajout latch sur adresse processor
-- 08/03/09 : Retour en arrire
C0 : iodecode port map ( RESETn_INT, BAOE_int, BAP, BAL, MAPn_INT, CSROMn_INT, CSRAMn_INT, CSIOn_INT );

C1 : addmemux port map ( RESETn_INT, VAP1, VAP2, BAP, VA1L, VA1R, VA1C, VA2L, VA2R, VA2C, BAC, 
                         BAL, AD_RAM_INT );

C2 : memmap   port map ( txthir_sel, dblhgt_sel, forcetxt, CPT_H, CPT_V, VAP1, chrowcnt, 
                         txthir_dec );

C3 : ctrlseq  port map ( RESETn_INT, CLK_24_INT, txthir_dec, isAttrib, RW_INT, CSRAMn_INT, 
                         CLK_1_INT, CLK_4_INT, CLK_PIXEL_INT,
                         VA1L, VA1R, VA1C, VA2L, VA2R, VA2C, BAC, BAL, RAS, CAS, MUX_INT, 
                         RW_RAM_INT,
                         attrib_dec, ld_reg_0, reld_reg, ldfrombus, databus_en, 
                         BAOE_int,
                         SRAM_CE, SRAM_OE, SRAM_WE, SRAM_LATCH
                        );

C4 : vag      port map ( CLK_1_INT, RESETn_INT, freq_sel, CPT_H, CPT_V, reload_sel, forcetxt, 
                         CLK_FLASH, COMPSYNC_INT, blankingn );

C5 : video    port map ( RESETn_INT, CLK_PIXEL_INT, CLK_FLASH,
                         -- delete 17/11/2009 flash_sel, 
								 blankingn, reload_sel, DATA_BUS_INT, attrib_dec, 
                         databus_en, ldfrombus, ld_reg_0, reld_reg, chrowcnt,
                         RGB_INT, freq_sel, txthir_sel, isAttrib, dblhgt_sel, VAP2 );

-- C6 : gen_clock port map ( RESETn_INT, CLK_12_EXT, CLK_24_INT, CLK_12_INT, CLK_PIXEL_INT );


-- INTERNAL ASSIGN
BAP          <= AD;
DATA_BUS_INT <= DB;
CLK_24_INT   <= CLK;
RESETn_INT   <= RESETn;
MAPn_INT     <= MAPn;
RW_INT       <= RW;

-- EXTERNAL ASSIGN
PHI2         <= CLK_1_INT;
CLK_12_EXT   <= CLK;
AD_RAM       <= AD_RAM_INT(15 downto 8);
CSIOn        <= CSIOn_INT;
CSROMn       <= CSROMn_INT;
CSRAMn       <= CSRAMn_INT;

-- MANAGE RAM DYNAMIQUE
RASn         <= RAS;
CASn         <= CAS;
MUX          <= MUX_INT;
RW_RAM       <= RW_RAM_INT;
l_AD_SRAM    <= AD_RAM_INT;

-- MANAGE RAM STATIQUE
AD_SRAM <= l_AD_SRAM;
OE_SRAM <= SRAM_OE;
CE_SRAM <= SRAM_CE;
WE_SRAM <= SRAM_WE;
LATCH_SRAM <= SRAM_LATCH;
CLK_4   <= CLK_4_INT;

-- VIDEO OUT
R       <= RGB_INT(0);
G       <= RGB_INT(1);
B       <= RGB_INT(2);
SYNC    <= COMPSYNC_INT;

end architecture RTL;
